/*
	SHARP MZ-2800 Emulator 'EmuZ-2800'
	Skelton for retropc emulator

	Author : Takeda.Toshiya
	Date   : 2007.08.13 -

	[ keyboard ]
*/

#include "keyboard.h"

void KEYBOARD::initialize()
{
	key_stat = emu->key_buffer();
	column = 0;
	vm->regist_frame_event(this);
}

void KEYBOARD::write_signal(int id, uint32 data, uint32 mask)
{
	if(id == SIG_KEYBOARD_COLUMN) {
		column = data & mask;	// from z80pio port a
		create_keystat();
	}
}

void KEYBOARD::event_frame()
{
	// update key status
	uint8 buf[256];
	_memcpy(buf, key_stat, sizeof(buf));
	
	if(!buf[0x10] && buf[0x21])	// PAGE UP -> F13
		buf[0x7c] = 0x80;
	if(!buf[0x10] && buf[0x22])	// PAGE UP -> F14
		buf[0x7d] = 0x80;
	if(buf[0x10] && buf[0x21])	// SHIFT + PAGE UP -> F15
		buf[0x7e] = 0x80;
	if(buf[0x10] && buf[0x22])	// SHIFT + PAGE DOWN -> F16
		buf[0x7f] = 0x80;
	if(buf[0x2d])			// INS -> SHIFT + DEL
		buf[0x10] = buf[0x2e] = 0x80;
	buf[0] = 0;
	
	keys[0xf] = 0xff;
	for(int i = 0; i <= 0xd; i++) {
		uint8 tmp = 0;
		for(int j = 0; j < 8; j++)
			tmp |= (buf[key_map[i][j]]) ? 0 : (1 << j);
		keys[i] = tmp;
		keys[0xf] &= tmp;
	}
	create_keystat();
}

void KEYBOARD::create_keystat()
{
	uint8 val = (!(column & 0x10)) ? keys[0xf] : ((column & 0xf) > 0xd) ? 0xff : keys[column & 0xf];
	d_pio0->write_signal(did_pio0, val, 0x80);	// to i8255 port b
	d_pio1->write_signal(did_pio1, val, 0xff);	// to z80pio port b
}

